@@ -903,6 +903,10 @@ char* gok_data_get_name_accessmethod ()
  */
 gboolean gok_data_get_wordcomplete ()
 {
+	/* don't use during login mode */
+	if (gok_main_get_login()) {
+		return FALSE;
+	}
 	return m_bWordCompleteOn;
 }
 
@@ -925,6 +929,10 @@ void gok_data_set_wordcomplete (gboolean bTrueFalse)
  */
 gboolean gok_data_get_commandprediction ()
 {
+	/* don't use during login mode */
+	if (gok_main_get_login()) {
+		return FALSE;
+	}
 	return TRUE;/* TODO m_bCommandPredictOn;*/
 }
 
