@@ -23,6 +23,7 @@
 #define __GOKDATA__H__
 
 #include <glib.h>
+#include <gconf/gconf-client.h>
 #include "gok-scanner.h"
 
 #ifdef __cplusplus
@@ -115,6 +116,9 @@ gdouble gok_data_get_valuator_sensitivity (void);
 void gok_data_set_valuator_sensitivity (gdouble multiplier);
 GokDockType gok_data_get_dock_type (void);
 void gok_data_set_dock_type (GokDockType dock_type);
+gint gok_data_get_repeat_rate (void);
+void gok_data_set_repeat_rate (gint rate);
+GConfClient* gok_data_get_gconf_client (void);
 
 #ifdef __cplusplus
 }
