@@ -33,6 +33,7 @@
 #include <gnome.h>
 #include "gok-spy-priv.h"
 #include "gok-output.h"
+#include "gok-word-complete.h"
 
 #ifdef __cplusplus
 extern "C" {
@@ -247,6 +248,7 @@ gint gok_key_get_effective_group (void);
 GokKeyImage* gok_keyimage_new (GokKey *pKey, gchar* Filename);
 void gok_keyimage_delete (GokKeyImage* pKeyImage);
 gboolean gok_key_isRepeatable(GokKey* pKey);
+GokOutput* gok_key_wordcomplete_output (GokKey *pKey, GokWordComplete *complete);
 
 #ifdef __cplusplus
 }
