@@ -39,15 +39,29 @@
 extern "C" {
 #endif /* __cplusplus */
 
+	
+/* keyboards can be assigned a both layout type and a shape type, eventually
+	there will be implementations for the various combinations */
+
 /* keyboard layouts */
 typedef enum {
 KEYBOARD_LAYOUT_NORMAL,
 KEYBOARD_LAYOUT_QWERTY,
 KEYBOARD_LAYOUT_CENTER,
 KEYBOARD_LAYOUT_UPPERL,
-KEYBOARD_LAYOUT_FITWINDOW
+KEYBOARD_LAYOUT_UNSPECIFIED
 } KeyboardLayouts;
 
+/* keyboard shapes */
+typedef enum {
+KEYBOARD_SHAPE_BEST,		/* use best shape for layout/access_method */
+KEYBOARD_SHAPE_KEYSQUARE,	/* good for switch scanning users */
+KEYBOARD_SHAPE_SQUARE,		/* good for pointer users */
+KEYBOARD_SHAPE_WIDE,		/* use when docked top or bottom */
+KEYBOARD_SHAPE_FITWINDOW,
+KEYBOARD_SHAPE_UNSPECIFIED
+} KeyboardShape;
+
 /* keyboard types */
 typedef enum {
 KEYBOARD_TYPE_PLAIN,
@@ -95,8 +109,9 @@ typedef enum {
 /* If you add data members to this structure, initialize them in gok_keyboard_new */
 typedef struct GokKeyboard {
 	gchar Name[MAX_KEYBOARD_NAME + 1];
-	gint LayoutType;
-	gint Type;
+	KeyboardShape shape;
+	KeyboardLayouts LayoutType;
+	KeyboardTypes Type;
 	gboolean bDynamicallyCreated;
 	gint NumberRows;
 	gint NumberColumns;
@@ -109,8 +124,8 @@ typedef struct GokKeyboard {
 	GokKeyboardExpandPolicy expand;
 	Accessible* pAccessible;
 	AccessibleNode* pNodeAccessible;
-        AccessibleRole  search_role;
-        GokSpySearchType search_type;
+	AccessibleRole  search_role;
+	GokSpySearchType search_type;
 	gboolean bFontCalculated;
 	struct GokKey* pKeyFirst;
 	struct GokKeyboard* pKeyboardNext;
@@ -124,6 +139,8 @@ int gok_xkb_base_event_type;
 GokKeyboard* gok_keyboard_new (void);
 GokKeyboard* gok_keyboard_read (gchar* Filename);
 GokKeyboard* gok_keyboard_get_core (void);
+GokKeyboard* gok_keyboard_get_alpha (void);
+GokKeyboard* gok_keyboard_get_alpha_by_frequency (void);
 Display*     gok_keyboard_get_display (void);
 void gok_keyboard_delete (GokKeyboard* pKeyboard, gboolean bForce);
 void gok_keyboard_delete_key (GokKey* pKey, GokKeyboard* pKeyboard);
@@ -150,8 +167,8 @@ GokKey* gok_key_new (GokKey* pKeyPrevious, GokKey* pKeyNext, GokKeyboard* pKeybo
 GokKey* gok_key_from_xkb_key (GokKey* prevKey, GokKeyboard *pKeyboard, Display *display, XkbGeometryPtr pGeom, XkbRowPtr pRow, XkbSectionPtr pXkbSection, XkbKeyPtr keyp, gint section, gint row, gint col);
 XkbDescPtr gok_keyboard_get_xkb_desc (void);
 gboolean gok_keyboard_xkb_select (Display *display);
-void     gok_keyboard_notify_xkb_event (XkbEvent* event);
-gboolean gok_keyboard_layout (GokKeyboard* pKeyboard);
+void gok_keyboard_notify_xkb_event (XkbEvent* event);
+gboolean gok_keyboard_layout (GokKeyboard* pKeyboard, KeyboardLayouts layout, KeyboardShape shape, gboolean force);
 gboolean gok_keyboard_branch_byKey (GokKey* pKey);
 gboolean gok_keyboard_branch_gui (AccessibleNode* pNodeAccessible, GokSpySearchType type, AccessibleRole role);
 gboolean gok_keyboard_branch_gui_actions (AccessibleNode* pNodeAccessible);
@@ -182,6 +199,7 @@ void gok_keyboard_update_labels (void);
 gint gok_keyboard_get_keyboards(void);
 GokKey *gok_keyboard_find_key_at (GokKeyboard *pKeyboard, gint x, gint y);
 GokKeyboardDirection gok_keyboard_parse_direction (const gchar *string);
+gint gok_keyboard_add_predictions (GokKeyboard *pKeyboard, gchar **list);
 
 #ifdef __cplusplus
 }
