@@ -1,4 +1,4 @@
-/* gok-output.h
+/* gok-output.c
 *
 * Copyright 2001,2002 Sun Microsystems, Inc.,
 * Copyright 2001,2002 University Of Toronto
@@ -31,6 +31,68 @@
 #include "gok-word-complete.h"
 #include "main.h"
 
+static gboolean
+gok_output_can_predict (void)
+{
+	GokKeyboard *kbd = gok_main_get_current_keyboard ();
+	return (kbd && !gok_main_get_login() && gok_data_get_wordcomplete () &&
+		gok_keyboard_get_supports_wordcomplete (kbd));
+}
+
+/**
+ * gok_output_update_predictions:
+ * Send new data to the predictor based on output type, and update the keyboard's prediction keys. 
+ **/
+static void
+gok_output_update_predictions (GokOutput    *output,
+			       gpointer      data)
+{
+	gunichar letter;
+	gchar *string;
+	gchar **prediction_list = NULL;
+
+	if (output) {
+		switch (output->Type) 
+		{
+		case OUTPUT_KEYCODE:
+		case OUTPUT_KEYSYM:
+			prediction_list = gok_wordcomplete_process_and_predict (gok_wordcomplete_get_default (),
+										(gunichar) data,
+										gok_data_get_num_predictions ());
+			gok_keyboard_set_predictions (gok_main_get_current_keyboard (),
+						      prediction_list);
+			break;
+		case OUTPUT_KEYSTRING:
+			/* TODO: Test! */
+			string = (gchar *) data;
+			if (string && g_utf8_validate (string, -1, NULL)) 
+			{
+				while (string) 
+				{
+					if (prediction_list)
+					{
+						int i;
+						for (i = 0; prediction_list && prediction_list[i]; ++i)
+							g_free (prediction_list[i]);
+						g_free (prediction_list);
+					}
+					letter = g_utf8_get_char (string);
+					prediction_list = gok_wordcomplete_process_and_predict (
+						gok_wordcomplete_get_default (),
+						letter,
+						gok_data_get_num_predictions ());
+					string = g_utf8_next_char (string);
+				}
+				gok_keyboard_set_predictions (gok_main_get_current_keyboard (), 
+							      prediction_list);
+			}
+			break;
+		default:
+			break;
+		}
+	}
+}
+
 gunichar gok_output_unichar_from_keysym (KeySym keysym)
 {
 	char *charstring;
@@ -211,9 +273,9 @@ void gok_output_send_to_system (GokOutput* pOutput, gboolean bWordCompletion)
 			SPI_generateKeyboardEvent ((long) keysym, NULL, SPI_KEY_SYM);
 
 			/* update the word prediction keys with the new output */
-			if (bWordCompletion && !gok_main_get_login()) {
-					gok_wordcomplete_predict (
-						gok_output_unichar_from_keysym (keysym));
+			if (bWordCompletion && gok_output_can_predict ()) {
+				gok_output_update_predictions (pOutput,
+							       (gpointer) gok_output_unichar_from_keysym (keysym));
 			}
 		}
 		else if (pOutput->Type == OUTPUT_KEYCODE)
@@ -225,8 +287,9 @@ void gok_output_send_to_system (GokOutput* pOutput, gboolean bWordCompletion)
 			gok_log ("pOutput->Flag = %d", (int) pOutput->Flag);
 
 			SPI_generateKeyboardEvent ((long) keycode, NULL, pOutput->Flag);
+
 			/* update the word prediction keys with the new output */
-			if (bWordCompletion && !gok_main_get_login() &&
+			if (bWordCompletion && gok_output_can_predict () &&
 			    !gok_key_modifier_for_keycode (
 				    gok_main_display (),
 				    gok_keyboard_get_xkb_desc (),
@@ -240,7 +303,8 @@ void gok_output_send_to_system (GokOutput* pOutput, gboolean bWordCompletion)
 				/* TODO: get the correct index from XKB */
 			        keysym = XKeycodeToKeysym (gok_main_display (), 
 							   keycode, index);
-				gok_wordcomplete_predict (gok_output_unichar_from_keysym (keysym));
+				gok_output_update_predictions (pOutput,
+							       (gpointer) gok_output_unichar_from_keysym (keysym));
 			}
 		}
 		else if (pOutput->Type == OUTPUT_KEYSTRING)
@@ -251,10 +315,10 @@ void gok_output_send_to_system (GokOutput* pOutput, gboolean bWordCompletion)
 			SPI_generateKeyboardEvent (0, pOutput->Name, SPI_KEY_STRING);
 
 			/* update the word prediction keys with the new output */
-			if (bWordCompletion == TRUE)
+			if (bWordCompletion == TRUE && gok_output_can_predict ())
 			{
-				/* TODO: allow word completion with substring insertion */
-				gok_wordcomplete_predict (0);
+				gok_output_update_predictions (pOutput,
+							       pOutput->Name);
 			}
 		}
 		else if (pOutput->Type == OUTPUT_EXEC)
