@@ -116,14 +116,14 @@ gboolean gok_page_wordcomplete_apply ()
 	{
 		bDataChanged = TRUE;
 		gok_data_set_wordcomplete (m_WordCompleteOnOff);
-		gok_wordcomplete_on (m_WordCompleteOnOff);
+		gok_keyslotter_on (m_WordCompleteOnOff, KEYTYPE_WORDCOMPLETE);
 	}
 
 	if (m_NumberPredictions != gok_data_get_num_predictions())
 	{
 		bDataChanged = TRUE;
 		gok_data_set_num_predictions (m_NumberPredictions);
-		gok_wordcomplete_change_number_predictions (m_NumberPredictions);
+		gok_keyslotter_change_number_predictions (m_NumberPredictions, KEYTYPE_WORDCOMPLETE);
 	}
 
 	return bDataChanged;
@@ -151,7 +151,7 @@ gboolean gok_page_wordcomplete_revert ()
 		bDataChanged = TRUE;
 		m_WordCompleteOnOff = m_WordCompleteOnOffBackup;
 		gok_data_set_wordcomplete (m_WordCompleteOnOff);
-		gok_wordcomplete_on (m_WordCompleteOnOff);
+		gok_keyslotter_on (m_WordCompleteOnOff, KEYTYPE_WORDCOMPLETE);
 
 		pCheckboxOnOff = lookup_widget (m_pWindowSettings, "checkWordCompletion");
 		g_assert (pCheckboxOnOff != NULL);
@@ -164,7 +164,7 @@ gboolean gok_page_wordcomplete_revert ()
 		bDataChanged = TRUE;
 		m_NumberPredictions = m_NumberPredictionsBackup;
 		gok_data_set_num_predictions (m_NumberPredictions);
-		gok_wordcomplete_change_number_predictions (m_NumberPredictions);
+		gok_keyslotter_change_number_predictions (m_NumberPredictions, KEYTYPE_WORDCOMPLETE);
 
 		pSpinNumberPredictions = lookup_widget (m_pWindowSettings, "spinNumberPredictions");
 		g_assert (pSpinNumberPredictions != NULL);
