@@ -43,6 +43,15 @@
 /* pointer to the settings dialog window */
 static GtkWidget* m_pWindowSettings;
 
+static gboolean is_locked;
+
+void
+gok_settingsdialog_lock (void)
+{
+	is_locked = TRUE;
+}
+
+
 /* Private functions */
 
 /**
@@ -95,6 +104,8 @@ gboolean gok_settingsdialog_open (gboolean bShow)
 		return FALSE;
 	}
 
+	gtk_button_box_set_layout (GTK_BUTTON_BOX (hbuttonbox), GTK_BUTTONBOX_END);
+
 #if GTK_MAJOR_VERSION < 2
 #if GTK_MINOR_VERSION < 3
 /* these calls can be used only on GTK 1.2 and below */
@@ -120,7 +131,22 @@ gboolean gok_settingsdialog_open (gboolean bShow)
 	pButtonHelp = gtk_button_new_from_stock (GTK_STOCK_HELP);
 #endif
 
-	gtk_button_set_label (GTK_BUTTON(pButtonTry), "Try");
+	gtk_widget_ref (pButtonHelp);
+	gtk_object_set_data_full (GTK_OBJECT (m_pWindowSettings), "pButtonHelp", pButtonHelp,
+	                            (GtkDestroyNotify) gtk_widget_unref);
+	gtk_widget_show (pButtonHelp);
+	gtk_box_pack_start (GTK_BOX (hbuttonbox), pButtonHelp, FALSE, FALSE, 10);
+#ifndef EARLY_GTK_VERSION
+	gtk_button_box_set_child_secondary (GTK_BUTTON_BOX (hbuttonbox), pButtonHelp, TRUE);
+#endif
+	GTK_WIDGET_SET_FLAGS (pButtonHelp, GTK_CAN_DEFAULT);
+	gtk_tooltips_set_tip (tooltips, pButtonHelp, _("display user help"), NULL);
+
+	gtk_signal_connect (GTK_OBJECT (pButtonHelp), "clicked",
+                      GTK_SIGNAL_FUNC (on_button_help),
+                      NULL);
+	
+	gtk_button_set_label (GTK_BUTTON(pButtonTry), "_Try");
 /*	gtk_button_set_use_stock (GTK_BUTTON(pButtonTry), TRUE); */
 
 	gtk_widget_ref (pButtonTry);
@@ -147,6 +173,19 @@ gboolean gok_settingsdialog_open (gboolean bShow)
                       GTK_SIGNAL_FUNC (on_button_revert),
                       NULL);
 
+	gtk_widget_ref (pButtonCancel);
+	gtk_object_set_data_full (GTK_OBJECT (m_pWindowSettings), "pButtonCancel", pButtonCancel,
+	                            (GtkDestroyNotify) gtk_widget_unref);
+	gtk_widget_show (pButtonCancel);
+	gtk_box_pack_start (GTK_BOX (hbuttonbox), pButtonCancel, FALSE, FALSE, 10);
+
+	GTK_WIDGET_SET_FLAGS (pButtonCancel, GTK_CAN_DEFAULT);
+	gtk_tooltips_set_tip (tooltips, pButtonCancel, _("cancel"), NULL);
+
+	gtk_signal_connect (GTK_OBJECT (pButtonCancel), "clicked",
+                      GTK_SIGNAL_FUNC (on_button_cancel),
+                      NULL);
+
 	gtk_widget_ref (pButtonOK);
 	gtk_object_set_data_full (GTK_OBJECT (m_pWindowSettings), "pButtonOK", pButtonOK,
 	                            (GtkDestroyNotify) gtk_widget_unref);
@@ -160,31 +199,6 @@ gboolean gok_settingsdialog_open (gboolean bShow)
                       GTK_SIGNAL_FUNC (on_button_ok),
                       NULL);
 
-	gtk_widget_ref (pButtonCancel);
-	gtk_object_set_data_full (GTK_OBJECT (m_pWindowSettings), "pButtonCancel", pButtonCancel,
-	                            (GtkDestroyNotify) gtk_widget_unref);
-	gtk_widget_show (pButtonCancel);
-	gtk_box_pack_start (GTK_BOX (hbuttonbox), pButtonCancel, FALSE, FALSE, 10);
-
-	GTK_WIDGET_SET_FLAGS (pButtonCancel, GTK_CAN_DEFAULT);
-	gtk_tooltips_set_tip (tooltips, pButtonCancel, _("cancel"), NULL);
-
-	gtk_signal_connect (GTK_OBJECT (pButtonCancel), "clicked",
-                      GTK_SIGNAL_FUNC (on_button_cancel),
-                      NULL);
-	
-	gtk_widget_ref (pButtonHelp);
-	gtk_object_set_data_full (GTK_OBJECT (m_pWindowSettings), "pButtonHelp", pButtonHelp,
-	                            (GtkDestroyNotify) gtk_widget_unref);
-	gtk_widget_show (pButtonHelp);
-	gtk_box_pack_start (GTK_BOX (hbuttonbox), pButtonHelp, FALSE, FALSE, 10);
-
-	GTK_WIDGET_SET_FLAGS (pButtonHelp, GTK_CAN_DEFAULT);
-	gtk_tooltips_set_tip (tooltips, pButtonHelp, _("display user help"), NULL);
-
-	gtk_signal_connect (GTK_OBJECT (pButtonHelp), "clicked",
-                      GTK_SIGNAL_FUNC (on_button_help),
-                      NULL);
 	
 	/* show/hide the settings dialog */
 	if (bShow == TRUE)
@@ -205,11 +219,20 @@ gboolean gok_settingsdialog_open (gboolean bShow)
 gboolean gok_settingsdialog_show()
 {
 	gboolean bFirstShowing;
-	
+	GtkWidget *label;
+
 	g_assert (m_pWindowSettings != NULL);
 
 	bFirstShowing = (GDK_IS_DRAWABLE(m_pWindowSettings->window) == FALSE) ? TRUE : FALSE;
 
+	/* if we've "locked" certain prefs, de-sensitize the relevant page tabs */
+	if (is_locked) {
+		label = lookup_widget (m_pWindowSettings, "label2");
+		gtk_widget_set_sensitive (label, FALSE);
+		label = lookup_widget (m_pWindowSettings, "label4");
+		gtk_widget_set_sensitive (label, FALSE);
+	}
+
 	gtk_widget_show (m_pWindowSettings);
 
 	/* if this is the first time we're showing the dialog then redisplay
@@ -253,10 +276,16 @@ GtkWidget* gok_settingsdialog_get_window ()
 **/
 void gok_settingsdialog_close()
 {
-	g_assert (m_pWindowSettings != NULL);
-	gtk_widget_destroy (m_pWindowSettings);
+	if (m_pWindowSettings != NULL)
+	{
+		gtk_widget_destroy (m_pWindowSettings);
 	
-	gok_page_accessmethod_close();
+		gok_page_accessmethod_close();
+	}
+	else
+	{
+		g_warning("The settings dialog doesn't exist in close function!");
+	}
 }
 
 /**
@@ -471,4 +500,3 @@ gint gok_settingsdialog_sort (gconstpointer pItem1, gconstpointer pItem2)
 {
 	return strcmp (pItem1, pItem2);
 }
-
