@@ -49,7 +49,7 @@ void gok_spy_open(void);        /* call this first (user must call SPI_init firs
 void gok_spy_close(void);       /* call this when shutting down */ 
 void gok_spy_stop (void);       /* call this first, as soon as you start shutting down */
  
-// listener support 
+/* listener support */
 void gok_spy_register_appchangelistener(AppChangeListener* callback); 
 void gok_spy_deregister_appchangelistener(AppChangeListener* callback); 
  
@@ -61,9 +61,10 @@ void gok_spy_deregister_mousebuttonlistener(MouseButtonListener* callback);
 
 gulong gok_spy_get_modmask (void);
 
+gboolean gok_spy_is_menu_role (AccessibleRole role);
 /* gboolean gok_spy_is_desktop(Accessible* pAccessible); */
-/* gboolean gok_spy_accessible_is_desktopChild(Accessible* accessible);*/
-/* AccessibleNode* gok_spy_get_applications( void ); */
+/* gboolean gok_spy_accessible_is_desktopChild(Accessible* accessible); */
+Accessible* gok_spy_get_active_frame( void );
 AccessibleNode* gok_spy_get_list( Accessible* paccessible, GokSpySearchType type, AccessibleRole role ); 
 AccessibleNode* gok_spy_get_ui_list( Accessible* paccessible); 
 AccessibleNode* gok_spy_refresh( AccessibleNode* plist); 
@@ -87,4 +88,4 @@ void gok_spy_accessible_unref(Accessible* accessible);
 } 
 #endif /* __cplusplus */ 
  
-#endif // #ifndef __GOKSPY_H__
+#endif /* #ifndef __GOKSPY_H__ */
