@@ -496,10 +496,10 @@ gok_log ("finished check");
 /* private */
 gboolean gok_main_display_scan_main()
 {
-	if (gok_main_display_scan ((GokKeyboard*)NULL, mainkeyboardname, KEYBOARD_TYPE_UNSPECIFIED, KEYBOARD_LAYOUT_NORMAL) == FALSE)
+	if (gok_main_display_scan ((GokKeyboard*)NULL, mainkeyboardname, KEYBOARD_TYPE_UNSPECIFIED, KEYBOARD_LAYOUT_NORMAL, KEYBOARD_SHAPE_BEST) == FALSE)
 	{
 		/* if no "main" keyboard then display first keyboard in list */
-		if (gok_main_display_scan ((GokKeyboard*)NULL, NULL, KEYBOARD_TYPE_UNSPECIFIED, KEYBOARD_LAYOUT_NORMAL) == FALSE)
+		if (gok_main_display_scan ((GokKeyboard*)NULL, NULL, KEYBOARD_TYPE_UNSPECIFIED, KEYBOARD_LAYOUT_NORMAL, KEYBOARD_SHAPE_BEST) == FALSE)
 		{
 			gok_main_display_error (_("No keyboards to display!"));
 			return FALSE;
@@ -641,8 +641,7 @@ void gok_main_window_change_listener (Accessible* pAccessible)
 		gok_scanner_start();
 	}
 
-	/* hack to make sure gok is not occluded - seems to be unstable*/
-	gdk_window_raise (m_pWindowMain->window);
+	gok_main_raise_window ();
 
 	gok_log_leave();	
 }
@@ -715,9 +714,8 @@ void gok_main_button_listener (gint button, gint state, long mods, long timestam
 
 	if (state == 0)
 	{
-		/* hack to make sure gok is not occluded - TODO: seems to be unstable*/
-		gdk_window_raise (m_pWindowMain->window);
-		
+		gok_main_raise_window ();
+
 		switch (button)
 		{
 			case 1:
@@ -768,6 +766,20 @@ void gok_main_button_listener (gint button, gint state, long mods, long timestam
 	gok_log_leave();	
 }
 
+/**
+ * gok_main_raise_window:
+ * hack to make sure gok is not occluded - TODO: seems to be unstable
+ *
+ **/
+void
+gok_main_raise_window (void)
+{
+	if (m_pWindowMain && m_pWindowMain->window) 
+	{
+		gdk_window_raise (m_pWindowMain->window);
+	}
+}
+
 /**
 * gok_main_mouse_button_listener
 * @button: Mouse button number that has changed state.
@@ -784,8 +796,7 @@ void gok_main_mousebutton_listener (gint button, gint state, long mods, long tim
 
 	if (state == 0)
 	{
-		/* hack to make sure gok is not occluded - TODO: seems to be unstable*/
-		gdk_window_raise (m_pWindowMain->window);
+		gok_main_raise_window ();
 
 		/* TODO: account for reversed button order, which is possible! 
 		 * i.e. leftbutton == 3, etc.
@@ -1044,9 +1055,10 @@ gok_main_keyboard_find_byname (const gchar *NameKeyboard)
 /**
 * gok_main_display_scan
 * @pKeyboard: If this is supplied it takes precedence over the name parameter (useful for dynamic keyboards with the same name)
-* @NameKeyboard: Name of the keyboard you want displayed (must be in the list of keyboards.)
-* @TypeKeyboard: Usually NORMAL but it can be used to specify a particular type of 
-* keyboard (Example: center-weighted or upper-left-weighted).
+* @nameKeyboard: Name of the keyboard you want displayed (must be in the list of keyboards.)
+# @typeKeyboard: Can be used to describe a runtime keyboard type (e.g. a Menus keyboard)
+* @layout: Can be used to specify a particular type of keyboard (Example: center-weighted or upper-left-weighted).
+* @shape: Can be used to specify a particular shape of keyboard (Example: square).
 *
 * Display a keyboard and allow the user to make selections from it.
 * The keyboard must have already been created and in the list of keyboards.
@@ -1056,8 +1068,11 @@ gok_main_keyboard_find_byname (const gchar *NameKeyboard)
 *
 * returns: TRUE if keyboard is displayed, FALSE if not.
 */
-gboolean gok_main_display_scan (GokKeyboard* pKeyboard, gchar* NameKeyboard, KeyboardTypes TypeKeyboard, KeyboardLayouts LayoutType)
+gboolean 
+gok_main_display_scan (GokKeyboard* pKeyboard, gchar* nameKeyboard, KeyboardTypes typeKeyboard, KeyboardLayouts layout, KeyboardShape shape)
 {
+	/* FIXME we're not using the type params */
+	
 	GokKeyboard* pKB;
 	gboolean bPushed;
 	gboolean bReturnCode;
@@ -1074,7 +1089,7 @@ gboolean gok_main_display_scan (GokKeyboard* pKeyboard, gchar* NameKeyboard, Key
 	}
 	else
 	{
-		pKB = gok_main_keyboard_find_byname (NameKeyboard);
+		pKB = gok_main_keyboard_find_byname (nameKeyboard);
 	}
 
 	if (pKB == NULL)
@@ -1126,8 +1141,7 @@ gboolean gok_main_ds (GokKeyboard* pKeyboard)
 	gok_scanner_stop();
 
 	/* raise the window as a precaution */
-	if (m_pWindowMain && m_pWindowMain->window)
-		gdk_window_raise (m_pWindowMain->window);
+	gok_main_raise_window ();
 
 	/* create new keys for dynamic keyboards */
 	if (pKeyboard->bDynamicallyCreated == TRUE)
@@ -1140,7 +1154,8 @@ gboolean gok_main_ds (GokKeyboard* pKeyboard)
 	}
 	
 	/* layout the keys on the keyboard */
-	if (gok_keyboard_layout (pKeyboard) == FALSE)
+	if (gok_keyboard_layout (pKeyboard, KEYBOARD_LAYOUT_NORMAL, 
+							KEYBOARD_SHAPE_SQUARE, FALSE) == FALSE)
 	{
 		gok_log_leave();
 		return FALSE;
@@ -1348,6 +1363,7 @@ GokKeyboard* gok_main_read_keyboards_from_dir (const char *directory)
 	gchar* complete_path;
 
 	gok_log_enter();
+	gok_log ("directory: %s", directory);
 	pKeyboardFirst = NULL;
 	pKeyboard = NULL;
 	pKeyboardPrevious = NULL;
@@ -1365,7 +1381,9 @@ GokKeyboard* gok_main_read_keyboards_from_dir (const char *directory)
 	/* create the default "core" keyboard (aka qwerty keyboard) */
 	if (gok_data_get_use_xkb_kbd ()) {
 		pKeyboard = pKeyboardFirst = gok_keyboard_get_core ();
-		pKeyboard->pKeyboardPrevious = NULL;
+		/* pKeyboard = pKeyboardFirst = gok_keyboard_get_alpha_by_frequency (); *//* TESTING ONLY!!!! */
+		if (pKeyboard)
+			pKeyboard->pKeyboardPrevious = NULL;
 		pKeyboardPrevious = pKeyboard;
 	}
 
@@ -1476,7 +1494,9 @@ void gok_main_initialize_wordcomplete ()
     }
     else
     {
-	if ( !(gok_wordcomplete_open (directory_name)) ) {
+	if ( !(gok_wordcomplete_open (gok_wordcomplete_get_default (),
+				      directory_name)) ||
+		! (gok_keyslotter_on (gok_wordcomplete_get_default (), KEYTYPE_WORDCOMPLETE))) {
 	    gok_log_x ("Error initializing word completion");
 	}
 	
@@ -1841,7 +1861,7 @@ void gok_main_close()
 	gok_action_close();
 	gok_feedback_close();
 	gok_control_button_callback_close();
-	gok_wordcomplete_close();
+	gok_wordcomplete_close(gok_wordcomplete_get_default ());
 	gok_predictor_close(m_pCommandPredictor);
 	
 	_exit(0);
