@@ -20,18 +20,45 @@
 *
 * To use this thing:
 * - Call "WordCompleteOpen". If it returns TRUE then you're ready to go.
-* - Call "WordCompletePredict" to make the word predictions.
+* - Call "WordCompletePredictString" to make the word predictions.
 * - Call "WordCompleteClose" when you're done. 
 * - To add a word, call "WordCompleteAddNewWord".
 *
 */
 
-#include "word-complete.h"
 #include <string.h>
 #include <stdlib.h>
 #include <ctype.h>
+#include <gnome.h>
+#include "word-complete.h"
 #include "gok-log.h"
 
+/* private functions */
+static gchar    WordCompleteSaveDictionary     (void);
+static gint     WordCompleteSaveLoop           (Node* pDictLetter, gchar* pWord, FILE* pFile);
+static void     WordCompletePredictLoop        (const gchar* pWord, Node* pNode, gchar* pPrediction);
+static void     WordCompleteDeleteChildren     (Node* pNode);
+static gboolean WordCompleteAddWord            (GokWordComplete *complete, 
+						const gchar* pWord, gint priority, gint state);
+static Node*    WordCompleteAddLetter          (Node* pNode, gchar letter);
+static void     WordCompleteAddPrediction      (gchar* pWord, gint priority);
+
+/* implementations of GokWordCompleteClass virtual methods */
+static gboolean WordCompleteAddNewWord         (GokWordComplete *complete, const gchar* pWord);
+static gboolean WordCompleteIncrementFrequency (GokWordComplete *complete, const gchar *pWord);
+static gboolean WordCompleteOpen               (GokWordComplete *complete, const gchar *dictionary);
+static void     WordCompleteClose              (GokWordComplete *complete);
+static gchar**  WordCompletePredictString      (GokWordComplete *complete, const gchar* pWord, gint numberPredictions);
+
+/* a prediction */
+struct PredictNode
+{
+	gchar word[MAXWORDLENGTH];
+	gint priority;
+};
+
+typedef struct PredictNode Prediction;
+
 typedef enum 
 {
 	WORD_CASE_LOWER,
@@ -59,6 +86,29 @@ static Prediction m_Predictions[MAXPREDICTIONS];
 /* The filename of the dictionary file */
 static gchar *dictionary_full_path;
 
+/* 
+ * This macro initializes GokTrieWordComplete with the GType system 
+ *   and defines ..._class_init and ..._instance_init functions 
+ */
+GNOME_CLASS_BOILERPLATE (GokTrieWordComplete, gok_trie_wordcomplete,
+			 GokWordComplete, GOK_TYPE_WORDCOMPLETE)
+
+static void
+gok_trie_wordcomplete_instance_init (GokTrieWordComplete *complete)
+{
+}
+
+static void
+gok_trie_wordcomplete_class_init (GokTrieWordCompleteClass *klass)
+{
+	GokWordCompleteClass *word_complete_class = (GokWordCompleteClass *) klass;
+	word_complete_class->open = WordCompleteOpen;
+	word_complete_class->close = WordCompleteClose;
+	word_complete_class->increment_word_frequency = WordCompleteIncrementFrequency;
+	word_complete_class->add_new_word = WordCompleteAddNewWord;
+	word_complete_class->predict_string = WordCompletePredictString;
+}
+
 /*
  * WordCompleteOpen
  *
@@ -73,7 +123,8 @@ static gchar *dictionary_full_path;
  * field 3 - the word's state
  * Example: "the 500 2"
  */
-char WordCompleteOpen(const gchar *directory)
+static gboolean
+WordCompleteOpen(GokWordComplete *complete, const gchar *directory)
 {
 	FILE *pFile;
 	gchar buffer[100];/* will hold each line of the file */
@@ -88,6 +139,7 @@ char WordCompleteOpen(const gchar *directory)
 	{
 		/* calling "Open" after it's already been opened! */
 		gok_log_x ("Calling 'Open' after WordComplete is already open!\n");
+		gok_wordcomplete_reset (complete); /* reset, anyhow */
 		return TRUE;
 	}
 
@@ -159,7 +211,7 @@ char WordCompleteOpen(const gchar *directory)
 		}
 
 		/* add it to the array of words */
-		WordCompleteAddWord (pWord, atoi (pPriority), atoi (pState));
+		WordCompleteAddWord (complete, pWord, atoi (pPriority), atoi (pState));
 	}
 
 	fclose (pFile);
@@ -174,7 +226,8 @@ char WordCompleteOpen(const gchar *directory)
  * Adds a word to the trie structure.
  * Returns TRUE if it was added, FALSE if not.
  */
-char WordCompleteAddWord (gchar* pWord, gint priority, gint state)
+static gboolean 
+WordCompleteAddWord (GokWordComplete *complete, const gchar* pWord, gint priority, gint state)
 {
 	/* this will eventually hold the final node (last letter) of the word */
 	Node* pNode;
@@ -215,7 +268,8 @@ char WordCompleteAddWord (gchar* pWord, gint priority, gint state)
  * FIXME: Not UTF-8 safe! letter should be a gunichar or 
  * utf-8 stringlet.
  */
-Node* WordCompleteAddLetter (Node* pNode, gchar letter)
+static Node* 
+WordCompleteAddLetter (Node* pNode, gchar letter)
 {
 	Node* pCurrentNode;
 	Node* pPreviousNode;
@@ -280,7 +334,8 @@ Node* WordCompleteAddLetter (Node* pNode, gchar letter)
  * Call this when done using the predictor.
  * If this is not called then memory leaks will result!
  */
-void WordCompleteClose()
+static void 
+WordCompleteClose(GokWordComplete *complete)
 {
 	Node* pDictLetter;
 	Node* pLastLetter;
@@ -316,7 +371,8 @@ void WordCompleteClose()
  * Deletes all the children of this node.
  * This is recursive so all the children's children are deleted too.
  */
-void WordCompleteDeleteChildren (Node* pParentNode)
+static void 
+WordCompleteDeleteChildren (Node* pParentNode)
 {
 	Node* pTempNode;
 	Node* pNode;
@@ -435,27 +491,21 @@ WordCompleteApplyCase (const gchar *pWord, gchar **pPredictions, gint num)
  * WordCompletePredict
  *
  * Makes word predictions based upon the part word given.
- * Returns an error code:
- * 0 - no error and at least one prediction has been made
- * 1 - no error but no predictions made
- * 2 - numberPredictions out of range
- * 3 - pWord is NULL
+ * Returns an array of strings which are the predictions, or NULL if no predictions were made.
  *
  * pWord - [in] the part word we want to complete
  * numberPredictions - [in] maximum number of predictions to return
- * pPredictions - [out] pointer to the string that holds the predictions
- *
- * Note: The char** "pPredictions" should NOT be freed by the caller.
- *
- * The predictions are returned as a string of words, separated by tabs.
- * Example: "the	there	then	thus"
- *
  */
-int WordCompletePredict (const gchar* pWord, gint numberPredictions, gchar** pPredictions)
+static gchar**
+WordCompletePredictString (GokWordComplete *complete, const gchar* pWord, gint numberPredictions)
 {
+	gchar **word_predict_list = NULL;
+
+	/* make the prediction */
+
 	gchar bufferLowercase[MAXWORDLENGTH + 1];
 	gint lengthWord;
-	gint count, x, y;
+	gint count, i, j;
 	gchar* pWordLowercase;
 	gchar buffer [MAXWORDLENGTH + 1];
 
@@ -470,19 +520,21 @@ int WordCompletePredict (const gchar* pWord, gint numberPredictions, gchar** pPr
 	/* validate the given values */
 	if (pWord == NULL)
 	{
-	  return 3;
+	  return NULL;
 	}
 
 	if ((numberPredictions < 1) ||
 	   (numberPredictions > MAXPREDICTIONS))
 	{
-	  return 2;
+	  return NULL;
 	}
 
+	word_predict_list = g_new0 (gchar *, numberPredictions);
+
 	/* initialize the array */
-	for (x = 0; x < MAXPREDICTIONS; x++)
+	for (i = 0; i < MAXPREDICTIONS; i++)
 	{
-	  m_Predictions[x].priority = 0;
+	  m_Predictions[i].priority = 0;
 	}
 
 	/* convert the given string to lowercase (cuz our dictionary is all lower case) */
@@ -514,17 +566,17 @@ int WordCompletePredict (const gchar* pWord, gint numberPredictions, gchar** pPr
 
 	/* look through our array of predictions and find the highest priority ones */
 	bMadePrediction = FALSE;
-	m_buffer[0] = 0;
-	for (x = 0; x < numberPredictions; x++)
+
+	for (i = 0; i < numberPredictions; i++)
 	{
 		highPriority = 0;
 		highPriorityIndex = -1;
-		for (y = 0; y < MAXPREDICTIONS; y++)
+		for (j = 0; j < MAXPREDICTIONS; j++)
 		{
-			if (m_Predictions[y].priority > highPriority)
+			if (m_Predictions[j].priority > highPriority)
 			{
-				highPriority = m_Predictions[y].priority;
-				highPriorityIndex = y;
+				highPriority = m_Predictions[j].priority;
+				highPriorityIndex = j;
 			}
 		}
 		if (highPriorityIndex == -1)
@@ -532,31 +584,23 @@ int WordCompletePredict (const gchar* pWord, gint numberPredictions, gchar** pPr
 		  break;
 		}
 
-		/* seperate the words by a tab character */
-		if (m_buffer[0] != 0)
-		{
-		  strcat (m_buffer, "\t");
-		}
-
 		/* add the prediction word to our buffer */
-		strcat (m_buffer, m_Predictions[highPriorityIndex].word);
+		word_predict_list[i] = strdup (m_Predictions[highPriorityIndex].word);
 		m_Predictions[highPriorityIndex].priority = 0;
 		bMadePrediction = TRUE;
 	}
 
-	/* return a pointer to our buffer that holds the string of predictions */
-	pPredictions[0] = m_buffer;
-
 	/* were there any predictions? */
 	if (bMadePrediction == FALSE)
 	{
-	  return 1; /* indicates no predictions made */
+		g_free (word_predict_list);
+ 	        return NULL; /* indicates no predictions made */
 	}
 
 	/* post-process the predictions to match the case of the input */
-	WordCompleteApplyCase (pWord, pPredictions, numberPredictions);
+	WordCompleteApplyCase (pWord, word_predict_list, numberPredictions);
 
-	return 0; /* predictions made! */
+	return word_predict_list; /* predictions made! */
 }
 
 /*
@@ -566,7 +610,8 @@ int WordCompletePredict (const gchar* pWord, gint numberPredictions, gchar** pPr
  * This function is recursive.
  * Each word is stored in an array by calling "AddPrediction".
  */
-void WordCompletePredictLoop (const gchar* pWordGiven, Node* pNode, gchar* pPrediction)
+static void 
+WordCompletePredictLoop (const gchar* pWordGiven, Node* pNode, gchar* pPrediction)
 {
 	if (pNode != NULL)
 	{
@@ -626,7 +671,8 @@ void WordCompletePredictLoop (const gchar* pWordGiven, Node* pNode, gchar* pPred
  * the predictions. After going through the entire trie, pick the highest
  * priority predictions from this array and return them to the caller.
  */
-void WordCompleteAddPrediction (gchar* pWord, gint priority)
+static void 
+WordCompleteAddPrediction (gchar* pWord, gint priority)
 {
     gint x;
     gint lowPriorityIndex;
@@ -672,11 +718,12 @@ void WordCompleteAddPrediction (gchar* pWord, gint priority)
  * WordCompleteAddNewWord
  *
  */
-char WordCompleteAddNewWord (gchar* pWord)
+static gboolean 
+WordCompleteAddNewWord (GokWordComplete *complete, const gchar* pWord)
 {
 	bWordCompleteModified = TRUE;
 
-	return WordCompleteAddWord (pWord, PRIORITY_NEWWORD, STATE_TEMPORARY);
+	return WordCompleteAddWord (complete, pWord, PRIORITY_NEWWORD, STATE_TEMPORARY);
 }
 
 static Node*
@@ -702,16 +749,14 @@ WordCompleteMatchingLeaf (Node *node, const gchar *pWord)
  * WordCompleteIncrementFrequency
  *
  */
-gboolean 
-WordCompleteIncrementFrequency (const gchar *pWord)
+static gboolean 
+WordCompleteIncrementFrequency (GokWordComplete *complete, const gchar *pWord)
 {
 	Node *node;
 	if (pWord) {
 		node = WordCompleteMatchingLeaf (&m_FirstNode, pWord);
-		fprintf (stderr, "node for %s: %p\n", pWord, node);
 		if (node && node->priority) {
 			++node->priority;
-			fprintf (stderr, "priority of %s is now %d\n", pWord, node->priority);
 			bWordCompleteModified = TRUE;
 			return TRUE;
 		}
@@ -725,7 +770,8 @@ WordCompleteIncrementFrequency (const gchar *pWord)
  * Saves the entire dictionary to disk.
  * Returns TRUE if saved OK, FALSE if dictionary was not saved.
  */
-char WordCompleteSaveDictionary()
+static char 
+WordCompleteSaveDictionary()
 {
 	FILE* pFile;
 	Node* pNode;
@@ -768,7 +814,8 @@ char WordCompleteSaveDictionary()
  *
  * Recursive loop for saving all dictionary words to disk.
  */
-int WordCompleteSaveLoop (Node* pNode, gchar* pBuffer, FILE* pFile)
+static int 
+WordCompleteSaveLoop (Node* pNode, gchar* pBuffer, FILE* pFile)
 {
     gint lengthWord;
     Node* pLetter;
